@@ -1,14 +1,15 @@
 package Net::DNS::Question;
 #
-# $Id: Question.pm,v 2.100 2003/12/13 01:37:05 ctriv Exp $
+# $Id: Question.pm 319 2005-05-30 17:12:09Z olaf $
 #
 use strict;
+use bytes;
 use vars qw($VERSION $AUTOLOAD);
 
 use Carp;
 use Net::DNS;
 
-$VERSION = (qw$Revision: 2.100 $)[1];
+$VERSION = (qw$LastChangedRevision: 319 $)[1];
 
 =head1 NAME
 
@@ -69,6 +70,9 @@ sub new {
 	bless \%self, $class;
 }
 
+
+
+
 #
 # Some people have reported that Net::DNS dies because AUTOLOAD picks up
 # calls to DESTROY.
@@ -181,7 +185,7 @@ sub data {
 
 Copyright (c) 1997-2002 Michael Fuhr. 
 
-Portions Copyright (c) 2002-2003 Chris Reinhardt.
+Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
