@@ -1,21 +1,19 @@
 package Net::DNS::RR::MB;
 #
-# $Id: MB.pm,v 2.100 2003/12/13 01:37:05 ctriv Exp $
+# $Id: MB.pm 319 2005-05-30 17:12:09Z olaf $
 #
 use strict;
+use bytes;
 use vars qw(@ISA $VERSION);
 
-use Net::DNS::Packet;
-
 @ISA     = qw(Net::DNS::RR);
-$VERSION = (qw$Revision: 2.100 $)[1];
+$VERSION = (qw$LastChangedRevision: 319 $)[1];
 
 sub new {
 	my ($class, $self, $data, $offset) = @_;
 
 	if ($self->{"rdlength"} > 0) {
-		my($madname) = Net::DNS::Packet::dn_expand($data, $offset);
-		$self->{"madname"} = $madname;
+		($self->{"madname"}) = Net::DNS::Packet::dn_expand($data, $offset);
 	}
 
 	return bless $self, $class;
@@ -35,9 +33,7 @@ sub new_from_string {
 sub rdatastr {
 	my $self = shift;
 
-	return exists $self->{"madname"}
-	       ? "$self->{madname}."
-	       : "; no data";
+	return $self->{"madname"} ? "$self->{madname}." : '';
 }
 
 sub rr_rdata {
@@ -59,6 +55,8 @@ sub _canonicalRdata {
 	}
 	return $rdata;
 }
+
+
 1;
 __END__
 
@@ -86,7 +84,7 @@ Returns the domain name of the host which has the specified mailbox.
 
 Copyright (c) 1997-2002 Michael Fuhr. 
 
-Portions Copyright (c) 2002-2003 Chris Reinhardt.
+Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
 All rights reserved.  This program is free software; you may redistribute
 it and/or modify it under the same terms as Perl itself.
